/*
 * Copyright 2020 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.acme.factoriolayout.rest;

import javax.inject.Inject;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;

import org.acme.factoriolayout.domain.FactorioLayout;
import org.acme.factoriolayout.persistence.FactorioLayoutRepository;
import org.optaplanner.core.api.score.ScoreManager;
import org.optaplanner.core.api.score.buildin.hardsoftlong.HardSoftLongScore;
import org.optaplanner.core.api.solver.SolverManager;
import org.optaplanner.core.api.solver.SolverStatus;

@Path("/factorioLayout")
@Produces(MediaType.APPLICATION_JSON)
@Consumes(MediaType.APPLICATION_JSON)
public class FactorioSolverResource {

    public static final Long SINGLETON_PROBLEM_ID = 1L;

    @Inject
    FactorioLayoutRepository factorioLayoutRepository;

    @Inject
    SolverManager<FactorioLayout, Long> solverManager;
    @Inject
    ScoreManager<FactorioLayout, HardSoftLongScore> scoreManager;

    @GET
    public FactorioLayout getFactorioLayout() {
        SolverStatus solverStatus = getSolverStatus();
        FactorioLayout factorioLayout = factorioLayoutRepository.get();
        scoreManager.updateScore(factorioLayout); // Sets the score
        factorioLayout.setSolverStatus(solverStatus);
        return factorioLayout;
    }

    @POST
    @Path("solve")
    public void solve() {
        solverManager.solveAndListen(SINGLETON_PROBLEM_ID,
                problemId -> factorioLayoutRepository.get(),
                factorioLayoutRepository::set);
    }

    public SolverStatus getSolverStatus() {
        return solverManager.getSolverStatus(SINGLETON_PROBLEM_ID);
    }

    @POST
    @Path("stopSolving")
    public void stopSolving() {
        solverManager.terminateEarly(SINGLETON_PROBLEM_ID);
    }

}
